@@ -1,29 +1,25 @@
 /*
- * Copyright (C) 2004-2008  Internet Systems Consortium, Inc. ("ISC")
- * Copyright (C) 2001-2003  Internet Software Consortium.
+ * Copyright (C) Internet Systems Consortium, Inc. ("ISC")
  *
- * Permission to use, copy, modify, and/or distribute this software for any
- * purpose with or without fee is hereby granted, provided that the above
- * copyright notice and this permission notice appear in all copies.
+ * This Source Code Form is subject to the terms of the Mozilla Public
+ * License, v. 2.0. If a copy of the MPL was not distributed with this
+ * file, You can obtain one at http://mozilla.org/MPL/2.0/.
  *
- * THE SOFTWARE IS PROVIDED "AS IS" AND ISC DISCLAIMS ALL WARRANTIES WITH
- * REGARD TO THIS SOFTWARE INCLUDING ALL IMPLIED WARRANTIES OF MERCHANTABILITY
- * AND FITNESS.  IN NO EVENT SHALL ISC BE LIABLE FOR ANY SPECIAL, DIRECT,
- * INDIRECT, OR CONSEQUENTIAL DAMAGES OR ANY DAMAGES WHATSOEVER RESULTING FROM
- * LOSS OF USE, DATA OR PROFITS, WHETHER IN AN ACTION OF CONTRACT, NEGLIGENCE
- * OR OTHER TORTIOUS ACTION, ARISING OUT OF OR IN CONNECTION WITH THE USE OR
- * PERFORMANCE OF THIS SOFTWARE.
+ * See the COPYRIGHT file distributed with this work for additional
+ * information regarding copyright ownership.
  */
 
-/* $Id: controlconf.c,v 1.60 2008-07-23 23:27:54 marka Exp $ */
-
 /*! \file */
 
 #include <config.h>
 
+#include <inttypes.h>
+#include <stdbool.h>
+
 #include <isc/base64.h>
 #include <isc/buffer.h>
 #include <isc/event.h>
+#include <isc/file.h>
 #include <isc/mem.h>
 #include <isc/net.h>
 #include <isc/netaddr.h>
@@ -71,6 +67,7 @@ typedef ISC_LIST(controllistener_t) controllistenerlist_t;
 
 struct controlkey {
 	char *				keyname;
+	uint32_t			algorithm;
 	isc_region_t			secret;
 	ISC_LINK(controlkey_t)		link;
 };
@@ -78,37 +75,38 @@ struct controlkey {
 struct controlconnection {
 	isc_socket_t *			sock;
 	isccc_ccmsg_t			ccmsg;
-	isc_boolean_t			ccmsg_valid;
-	isc_boolean_t			sending;
+	bool			ccmsg_valid;
+	bool			sending;
 	isc_timer_t *			timer;
-	unsigned char			buffer[2048];
+	isc_buffer_t *			buffer;
 	controllistener_t *		listener;
-	isc_uint32_t			nonce;
+	uint32_t			nonce;
 	ISC_LINK(controlconnection_t)	link;
 };
 
 struct controllistener {
-	ns_controls_t *			controls;
+	named_controls_t *		controls;
 	isc_mem_t *			mctx;
 	isc_task_t *			task;
 	isc_sockaddr_t			address;
 	isc_socket_t *			sock;
 	dns_acl_t *			acl;
-	isc_boolean_t			listening;
-	isc_boolean_t			exiting;
+	bool			listening;
+	bool			exiting;
 	controlkeylist_t		keys;
 	controlconnectionlist_t		connections;
 	isc_sockettype_t		type;
-	isc_uint32_t			perm;
-	isc_uint32_t			owner;
-	isc_uint32_t			group;
+	uint32_t			perm;
+	uint32_t			owner;
+	uint32_t			group;
+	bool			readonly;
 	ISC_LINK(controllistener_t)	link;
 };
 
-struct ns_controls {
-	ns_server_t			*server;
+struct named_controls {
+	named_server_t			*server;
 	controllistenerlist_t 		listeners;
-	isc_boolean_t			shuttingdown;
+	bool			shuttingdown;
 	isccc_symtab_t			*symtab;
 };
 
@@ -149,7 +147,7 @@ free_listener(controllistener_t *listener) {
 	if (listener->acl != NULL)
 		dns_acl_detach(&listener->acl);
 
-	isc_mem_put(listener->mctx, listener, sizeof(*listener));
+	isc_mem_putanddetach(&listener->mctx, listener, sizeof(*listener));
 }
 
 static void
@@ -164,6 +162,9 @@ static void
 maybe_free_connection(controlconnection_t *conn) {
 	controllistener_t *listener = conn->listener;
 
+	if (conn->buffer != NULL)
+		isc_buffer_free(&conn->buffer);
+
 	if (conn->timer != NULL)
 		isc_timer_detach(&conn->timer);
 
@@ -179,6 +180,11 @@ maybe_free_connection(controlconnection_t *conn) {
 	}
 
 	ISC_LIST_UNLINK(listener->connections, conn, link);
+#ifdef ENABLE_AFL
+	if (named_g_fuzz_type == isc_fuzz_rndc) {
+		named_fuzz_notify();
+	}
+#endif
 	isc_mem_put(listener->mctx, conn, sizeof(*conn));
 }
 
@@ -194,12 +200,12 @@ shutdown_listener(controllistener_t *listener) {
 
 		isc_sockaddr_format(&listener->address, socktext,
 				    sizeof(socktext));
-		isc_log_write(ns_g_lctx, NS_LOGCATEGORY_GENERAL,
-			      NS_LOGMODULE_CONTROL, ISC_LOG_NOTICE,
+		isc_log_write(named_g_lctx, NAMED_LOGCATEGORY_GENERAL,
+			      NAMED_LOGMODULE_CONTROL, ISC_LOG_NOTICE,
 			      "stopping command channel on %s", socktext);
 		if (listener->type == isc_sockettype_unix)
-			isc_socket_cleanunix(&listener->address, ISC_TRUE);
-		listener->exiting = ISC_TRUE;
+			isc_socket_cleanunix(&listener->address, true);
+		listener->exiting = true;
 	}
 
 	for (conn = ISC_LIST_HEAD(listener->connections);
@@ -217,21 +223,22 @@ shutdown_listener(controllistener_t *listener) {
 	maybe_free_listener(listener);
 }
 
-static isc_boolean_t
+static bool
 address_ok(isc_sockaddr_t *sockaddr, dns_acl_t *acl) {
+	dns_aclenv_t *env =
+		ns_interfacemgr_getaclenv(named_g_server->interfacemgr);
 	isc_netaddr_t netaddr;
 	isc_result_t result;
 	int match;
 
 	isc_netaddr_fromsockaddr(&netaddr, sockaddr);
 
-	result = dns_acl_match(&netaddr, NULL, acl,
-			       &ns_g_server->aclenv, &match, NULL);
+	result = dns_acl_match(&netaddr, NULL, acl, env, &match, NULL);
 
 	if (result != ISC_R_SUCCESS || match <= 0)
-		return (ISC_FALSE);
+		return (false);
 	else
-		return (ISC_TRUE);
+		return (true);
 }
 
 static isc_result_t
@@ -245,7 +252,7 @@ control_accept(controllistener_t *listener) {
 				 "isc_socket_accept() failed: %s",
 				 isc_result_totext(result));
 	else
-		listener->listening = ISC_TRUE;
+		listener->listening = true;
 	return (result);
 }
 
@@ -278,7 +285,7 @@ control_senddone(isc_task_t *task, isc_event_t *event) {
 
 	UNUSED(task);
 
-	conn->sending = ISC_FALSE;
+	conn->sending = false;
 
 	if (sevent->result != ISC_R_SUCCESS &&
 	    sevent->result != ISC_R_CANCELED)
@@ -288,8 +295,8 @@ control_senddone(isc_task_t *task, isc_event_t *event) {
 
 		(void)isc_socket_getpeername(sock, &peeraddr);
 		isc_sockaddr_format(&peeraddr, socktext, sizeof(socktext));
-		isc_log_write(ns_g_lctx, NS_LOGCATEGORY_GENERAL,
-			      NS_LOGMODULE_CONTROL, ISC_LOG_WARNING,
+		isc_log_write(named_g_lctx, NAMED_LOGCATEGORY_GENERAL,
+			      NAMED_LOGMODULE_CONTROL, ISC_LOG_WARNING,
 			      "error sending command response to %s: %s",
 			      socktext, isc_result_totext(sevent->result));
 	}
@@ -311,39 +318,42 @@ log_invalid(isccc_ccmsg_t *ccmsg, isc_result_t result) {
 
 	(void)isc_socket_getpeername(ccmsg->sock, &peeraddr);
 	isc_sockaddr_format(&peeraddr, socktext, sizeof(socktext));
-	isc_log_write(ns_g_lctx, NS_LOGCATEGORY_GENERAL,
-		      NS_LOGMODULE_CONTROL, ISC_LOG_ERROR,
+	isc_log_write(named_g_lctx, NAMED_LOGCATEGORY_GENERAL,
+		      NAMED_LOGMODULE_CONTROL, ISC_LOG_ERROR,
 		      "invalid command from %s: %s",
 		      socktext, isc_result_totext(result));
 }
 
 static void
 control_recvmessage(isc_task_t *task, isc_event_t *event) {
-	controlconnection_t *conn;
-	controllistener_t *listener;
-	controlkey_t *key;
+	controlconnection_t *conn = NULL;
+	controllistener_t *listener = NULL;
+	named_server_t *server = NULL;
+	controlkey_t *key = NULL;
 	isccc_sexpr_t *request = NULL;
 	isccc_sexpr_t *response = NULL;
-	isccc_region_t ccregion;
+	uint32_t algorithm;
 	isccc_region_t secret;
 	isc_stdtime_t now;
 	isc_buffer_t b;
 	isc_region_t r;
-	isc_uint32_t len;
-	isc_buffer_t text;
-	char textarray[1024];
+	isc_buffer_t *text;
 	isc_result_t result;
 	isc_result_t eresult;
-	isccc_sexpr_t *_ctrl;
+	isccc_sexpr_t *_ctrl = NULL;
 	isccc_time_t sent;
 	isccc_time_t exp;
-	isc_uint32_t nonce;
+	uint32_t nonce;
+	isccc_sexpr_t *data = NULL;
 
 	REQUIRE(event->ev_type == ISCCC_EVENT_CCMSG);
 
 	conn = event->ev_arg;
 	listener = conn->listener;
+	server = listener->controls->server;
+	algorithm = DST_ALG_UNKNOWN;
 	secret.rstart = NULL;
+	text = NULL;
 
 	/* Is the server shutting down? */
 	if (listener->controls->shuttingdown)
@@ -362,25 +372,22 @@ control_recvmessage(isc_task_t *task, isc_event_t *event) {
 	     key != NULL;
 	     key = ISC_LIST_NEXT(key, link))
 	{
+		isccc_region_t ccregion;
+
 		ccregion.rstart = isc_buffer_base(&conn->ccmsg.buffer);
 		ccregion.rend = isc_buffer_used(&conn->ccmsg.buffer);
 		secret.rstart = isc_mem_get(listener->mctx, key->secret.length);
 		if (secret.rstart == NULL)
 			goto cleanup;
-		memcpy(secret.rstart, key->secret.base, key->secret.length);
+		memmove(secret.rstart, key->secret.base, key->secret.length);
 		secret.rend = secret.rstart + key->secret.length;
-		result = isccc_cc_fromwire(&ccregion, &request, &secret);
+		algorithm = key->algorithm;
+		result = isccc_cc_fromwire(&ccregion, &request,
+					   algorithm, &secret);
 		if (result == ISC_R_SUCCESS)
 			break;
 		isc_mem_put(listener->mctx, secret.rstart, REGION_SIZE(secret));
-		if (result == ISCCC_R_BADAUTH) {
-			/*
-			 * For some reason, request is non-NULL when
-			 * isccc_cc_fromwire returns ISCCC_R_BADAUTH.
-			 */
-			if (request != NULL)
-				isccc_sexpr_free(&request);
-		} else {
+		if (result != ISCCC_R_BADAUTH) {
 			log_invalid(&conn->ccmsg, result);
 			goto cleanup;
 		}
@@ -403,7 +410,7 @@ control_recvmessage(isc_task_t *task, isc_event_t *event) {
 	 * Limit exposure to replay attacks.
 	 */
 	_ctrl = isccc_alist_lookup(request, "_ctrl");
-	if (_ctrl == NULL) {
+	if (!isccc_alist_alistp(_ctrl)) {
 		log_invalid(&conn->ccmsg, ISC_R_FAILURE);
 		goto cleanup_request;
 	}
@@ -446,22 +453,34 @@ control_recvmessage(isc_task_t *task, isc_event_t *event) {
 		goto cleanup_request;
 	}
 
+	result = isc_buffer_allocate(listener->mctx, &text, 2 * 2048);
+	if (result != ISC_R_SUCCESS)
+		goto cleanup_request;
+
 	/*
 	 * Establish nonce.
 	 */
-	while (conn->nonce == 0)
-		isc_random_get(&conn->nonce);
-
-	isc_buffer_init(&text, textarray, sizeof(textarray));
-	eresult = ns_control_docommand(request, &text);
+	if (conn->nonce == 0) {
+		while (conn->nonce == 0) {
+			isc_rng_randombytes(server->sctx->rngctx, &conn->nonce,
+					    sizeof(conn->nonce));
+		}
+		eresult = ISC_R_SUCCESS;
+	} else
+		eresult = named_control_docommand(request, listener->readonly,
+						  &text);
 
 	result = isccc_cc_createresponse(request, now, now + 60, &response);
 	if (result != ISC_R_SUCCESS)
 		goto cleanup_request;
-	if (eresult != ISC_R_SUCCESS) {
-		isccc_sexpr_t *data;
 
-		data = isccc_alist_lookup(response, "_data");
+	data = isccc_alist_lookup(response, "_data");
+	if (data != NULL) {
+		if (isccc_cc_defineuint32(data, "result", eresult) == NULL)
+			goto cleanup_response;
+	}
+
+	if (eresult != ISC_R_SUCCESS) {
 		if (data != NULL) {
 			const char *estr = isc_result_totext(eresult);
 			if (isccc_cc_definestring(data, "err", estr) == NULL)
@@ -469,12 +488,9 @@ control_recvmessage(isc_task_t *task, isc_event_t *event) {
 		}
 	}
 
-	if (isc_buffer_usedlength(&text) > 0) {
-		isccc_sexpr_t *data;
-
-		data = isccc_alist_lookup(response, "_data");
+	if (isc_buffer_usedlength(text) > 0) {
 		if (data != NULL) {
-			char *str = (char *)isc_buffer_base(&text);
+			char *str = (char *)isc_buffer_base(text);
 			if (isccc_cc_definestring(data, "text", str) == NULL)
 				goto cleanup_response;
 		}
@@ -485,25 +501,36 @@ control_recvmessage(isc_task_t *task, isc_event_t *event) {
 	    isccc_cc_defineuint32(_ctrl, "_nonce", conn->nonce) == NULL)
 		goto cleanup_response;
 
-	ccregion.rstart = conn->buffer + 4;
-	ccregion.rend = conn->buffer + sizeof(conn->buffer);
-	result = isccc_cc_towire(response, &ccregion, &secret);
+	if (conn->buffer == NULL) {
+		result = isc_buffer_allocate(listener->mctx,
+					     &conn->buffer, 2 * 2048);
+		if (result != ISC_R_SUCCESS)
+			goto cleanup_response;
+	}
+
+	isc_buffer_clear(conn->buffer);
+	/* Skip the length field (4 bytes) */
+	isc_buffer_add(conn->buffer, 4);
+
+	result = isccc_cc_towire(response, &conn->buffer, algorithm, &secret);
 	if (result != ISC_R_SUCCESS)
 		goto cleanup_response;
-	isc_buffer_init(&b, conn->buffer, 4);
-	len = sizeof(conn->buffer) - REGION_SIZE(ccregion);
-	isc_buffer_putuint32(&b, len - 4);
-	r.base = conn->buffer;
-	r.length = len;
+
+	isc_buffer_init(&b, conn->buffer->base, 4);
+	isc_buffer_putuint32(&b, conn->buffer->used - 4);
+
+	r.base = conn->buffer->base;
+	r.length = conn->buffer->used;
 
 	result = isc_socket_send(conn->sock, &r, task, control_senddone, conn);
 	if (result != ISC_R_SUCCESS)
 		goto cleanup_response;
-	conn->sending = ISC_TRUE;
+	conn->sending = true;
 
 	isc_mem_put(listener->mctx, secret.rstart, REGION_SIZE(secret));
 	isccc_sexpr_free(&request);
 	isccc_sexpr_free(&response);
+	isc_buffer_free(&text);
 	return;
 
  cleanup_response:
@@ -512,11 +539,13 @@ control_recvmessage(isc_task_t *task, isc_event_t *event) {
  cleanup_request:
 	isccc_sexpr_free(&request);
 	isc_mem_put(listener->mctx, secret.rstart, REGION_SIZE(secret));
+	if (text != NULL)
+		isc_buffer_free(&text);
 
  cleanup:
 	isc_socket_detach(&conn->sock);
 	isccc_ccmsg_invalidate(&conn->ccmsg);
-	conn->ccmsg_valid = ISC_FALSE;
+	conn->ccmsg_valid = false;
 	maybe_free_connection(conn);
 	maybe_free_listener(listener);
 }
@@ -545,11 +574,16 @@ newconnection(controllistener_t *listener, isc_socket_t *sock) {
 
 	conn->sock = sock;
 	isccc_ccmsg_init(listener->mctx, sock, &conn->ccmsg);
-	conn->ccmsg_valid = ISC_TRUE;
-	conn->sending = ISC_FALSE;
+
+	/* Set a 32 KiB upper limit on incoming message. */
+	isccc_ccmsg_setmaxsize(&conn->ccmsg, 32768);
+
+	conn->ccmsg_valid = true;
+	conn->sending = false;
+	conn->buffer = NULL;
 	conn->timer = NULL;
 	isc_interval_set(&interval, 60, 0);
-	result = isc_timer_create(ns_g_timermgr, isc_timertype_once,
+	result = isc_timer_create(named_g_timermgr, isc_timertype_once,
 				  NULL, &interval, listener->task,
 				  control_timeout, conn, &conn->timer);
 	if (result != ISC_R_SUCCESS)
@@ -563,16 +597,22 @@ newconnection(controllistener_t *listener, isc_socket_t *sock) {
 					 control_recvmessage, conn);
 	if (result != ISC_R_SUCCESS)
 		goto cleanup;
-	isccc_ccmsg_setmaxsize(&conn->ccmsg, 2048);
 
 	ISC_LIST_APPEND(listener->connections, conn, link);
 	return (ISC_R_SUCCESS);
 
  cleanup:
+	if (conn->buffer != NULL)
+		isc_buffer_free(&conn->buffer);
 	isccc_ccmsg_invalidate(&conn->ccmsg);
 	if (conn->timer != NULL)
 		isc_timer_detach(&conn->timer);
 	isc_mem_put(listener->mctx, conn, sizeof(*conn));
+#ifdef ENABLE_AFL
+	if (named_g_fuzz_type == isc_fuzz_rndc) {
+		named_fuzz_notify();
+	}
+#endif
 	return (result);
 }
 
@@ -586,7 +626,7 @@ control_newconn(isc_task_t *task, isc_event_t *event) {
 
 	UNUSED(task);
 
-	listener->listening = ISC_FALSE;
+	listener->listening = false;
 
 	if (nevent->result != ISC_R_SUCCESS) {
 		if (nevent->result == ISC_R_CANCELED) {
@@ -603,8 +643,8 @@ control_newconn(isc_task_t *task, isc_event_t *event) {
 	    !address_ok(&peeraddr, listener->acl)) {
 		char socktext[ISC_SOCKADDR_FORMATSIZE];
 		isc_sockaddr_format(&peeraddr, socktext, sizeof(socktext));
-		isc_log_write(ns_g_lctx, NS_LOGCATEGORY_GENERAL,
-			      NS_LOGMODULE_CONTROL, ISC_LOG_WARNING,
+		isc_log_write(named_g_lctx, NAMED_LOGCATEGORY_GENERAL,
+			      NAMED_LOGMODULE_CONTROL, ISC_LOG_WARNING,
 			      "rejected command channel message from %s",
 			      socktext);
 		isc_socket_detach(&sock);
@@ -615,8 +655,8 @@ control_newconn(isc_task_t *task, isc_event_t *event) {
 	if (result != ISC_R_SUCCESS) {
 		char socktext[ISC_SOCKADDR_FORMATSIZE];
 		isc_sockaddr_format(&peeraddr, socktext, sizeof(socktext));
-		isc_log_write(ns_g_lctx, NS_LOGCATEGORY_GENERAL,
-			      NS_LOGMODULE_CONTROL, ISC_LOG_WARNING,
+		isc_log_write(named_g_lctx, NAMED_LOGCATEGORY_GENERAL,
+			      NAMED_LOGMODULE_CONTROL, ISC_LOG_WARNING,
 			      "dropped command channel from %s: %s",
 			      socktext, isc_result_totext(result));
 		isc_socket_detach(&sock);
@@ -630,7 +670,7 @@ control_newconn(isc_task_t *task, isc_event_t *event) {
 }
 
 static void
-controls_shutdown(ns_controls_t *controls) {
+controls_shutdown(named_controls_t *controls) {
 	controllistener_t *listener;
 	controllistener_t *next;
 
@@ -648,9 +688,9 @@ controls_shutdown(ns_controls_t *controls) {
 }
 
 void
-ns_controls_shutdown(ns_controls_t *controls) {
+named_controls_shutdown(named_controls_t *controls) {
 	controls_shutdown(controls);
-	controls->shuttingdown = ISC_TRUE;
+	controls->shuttingdown = true;
 }
 
 static isc_result_t
@@ -700,6 +740,7 @@ controlkeylist_fromcfg(const cfg_obj_t *keylist, isc_mem_t *mctx,
 		if (key == NULL)
 			goto cleanup;
 		key->keyname = newstr;
+		key->algorithm = DST_ALG_UNKNOWN;
 		key->secret.base = NULL;
 		key->secret.length = 0;
 		ISC_LINK_INIT(key, link);
@@ -733,7 +774,7 @@ register_keys(const cfg_obj_t *control, const cfg_obj_t *keylist,
 
 		result = cfgkeylist_find(keylist, keyid->keyname, &keydef);
 		if (result != ISC_R_SUCCESS) {
-			cfg_obj_log(control, ns_g_lctx, ISC_LOG_WARNING,
+			cfg_obj_log(control, named_g_lctx, ISC_LOG_WARNING,
 				    "couldn't find key '%s' for use with "
 				    "command channel %s",
 				    keyid->keyname, socktext);
@@ -744,6 +785,7 @@ register_keys(const cfg_obj_t *control, const cfg_obj_t *keylist,
 			const cfg_obj_t *secretobj = NULL;
 			const char *algstr = NULL;
 			const char *secretstr = NULL;
+			unsigned int algtype;
 
 			(void)cfg_map_get(keydef, "algorithm", &algobj);
 			(void)cfg_map_get(keydef, "secret", &secretobj);
@@ -752,10 +794,10 @@ register_keys(const cfg_obj_t *control, const cfg_obj_t *keylist,
 			algstr = cfg_obj_asstring(algobj);
 			secretstr = cfg_obj_asstring(secretobj);
 
-			if (ns_config_getkeyalgorithm(algstr, NULL, NULL) !=
-			    ISC_R_SUCCESS)
+			if (named_config_getkeyalgorithm2(algstr, NULL,
+					  &algtype, NULL) != ISC_R_SUCCESS)
 			{
-				cfg_obj_log(control, ns_g_lctx,
+				cfg_obj_log(control, named_g_lctx,
 					    ISC_LOG_WARNING,
 					    "unsupported algorithm '%s' in "
 					    "key '%s' for use with command "
@@ -766,11 +808,13 @@ register_keys(const cfg_obj_t *control, const cfg_obj_t *keylist,
 				continue;
 			}
 
+			keyid->algorithm = algtype;
 			isc_buffer_init(&b, secret, sizeof(secret));
 			result = isc_base64_decodestring(secretstr, &b);
 
 			if (result != ISC_R_SUCCESS) {
-				cfg_obj_log(keydef, ns_g_lctx, ISC_LOG_WARNING,
+				cfg_obj_log(keydef, named_g_lctx,
+					    ISC_LOG_WARNING,
 					    "secret for key '%s' on "
 					    "command channel %s: %s",
 					    keyid->keyname, socktext,
@@ -784,15 +828,16 @@ register_keys(const cfg_obj_t *control, const cfg_obj_t *keylist,
 			keyid->secret.base = isc_mem_get(mctx,
 							 keyid->secret.length);
 			if (keyid->secret.base == NULL) {
-				cfg_obj_log(keydef, ns_g_lctx, ISC_LOG_WARNING,
+				cfg_obj_log(keydef, named_g_lctx,
+					    ISC_LOG_WARNING,
 					   "couldn't register key '%s': "
 					   "out of memory", keyid->keyname);
 				ISC_LIST_UNLINK(*keyids, keyid, link);
 				free_controlkey(keyid, mctx);
 				break;
 			}
-			memcpy(keyid->secret.base, isc_buffer_base(&b),
-			       keyid->secret.length);
+			memmove(keyid->secret.base, isc_buffer_base(&b),
+				keyid->secret.length);
 		}
 	}
 }
@@ -816,10 +861,18 @@ get_rndckey(isc_mem_t *mctx, controlkeylist_t *keyids) {
 	const char *secretstr = NULL;
 	controlkey_t *keyid = NULL;
 	char secret[1024];
+	unsigned int algtype;
 	isc_buffer_t b;
 
-	CHECK(cfg_parser_create(mctx, ns_g_lctx, &pctx));
-	CHECK(cfg_parse_file(pctx, ns_g_keyfile, &cfg_type_rndckey, &config));
+	isc_log_write(named_g_lctx, NAMED_LOGCATEGORY_GENERAL,
+		      NAMED_LOGMODULE_CONTROL, ISC_LOG_INFO,
+		      "configuring command channel from '%s'",
+		      named_g_keyfile);
+	if (! isc_file_exists(named_g_keyfile))
+		return (ISC_R_FILENOTFOUND);
+
+	CHECK(cfg_parser_create(mctx, named_g_lctx, &pctx));
+	CHECK(cfg_parse_file(pctx, named_g_keyfile, &cfg_type_rndckey, &config));
 	CHECK(cfg_map_get(config, "key", &key));
 
 	keyid = isc_mem_get(mctx, sizeof(*keyid));
@@ -829,11 +882,12 @@ get_rndckey(isc_mem_t *mctx, controlkeylist_t *keyids) {
 					cfg_obj_asstring(cfg_map_getname(key)));
 	keyid->secret.base = NULL;
 	keyid->secret.length = 0;
+	keyid->algorithm = DST_ALG_UNKNOWN;
 	ISC_LINK_INIT(keyid, link);
 	if (keyid->keyname == NULL)
 		CHECK(ISC_R_NOMEMORY);
 
-	CHECK(bind9_check_key(key, ns_g_lctx));
+	CHECK(bind9_check_key(key, named_g_lctx));
 
 	(void)cfg_map_get(key, "algorithm", &algobj);
 	(void)cfg_map_get(key, "secret", &secretobj);
@@ -842,8 +896,9 @@ get_rndckey(isc_mem_t *mctx, controlkeylist_t *keyids) {
 	algstr = cfg_obj_asstring(algobj);
 	secretstr = cfg_obj_asstring(secretobj);
 
-	if (ns_config_getkeyalgorithm(algstr, NULL, NULL) != ISC_R_SUCCESS) {
-		cfg_obj_log(key, ns_g_lctx,
+	if (named_config_getkeyalgorithm2(algstr, NULL,
+					  &algtype, NULL) != ISC_R_SUCCESS) {
+		cfg_obj_log(key, named_g_lctx,
 			    ISC_LOG_WARNING,
 			    "unsupported algorithm '%s' in "
 			    "key '%s' for use with command "
@@ -852,27 +907,28 @@ get_rndckey(isc_mem_t *mctx, controlkeylist_t *keyids) {
 		goto cleanup;
 	}
 
+	keyid->algorithm = algtype;
 	isc_buffer_init(&b, secret, sizeof(secret));
 	result = isc_base64_decodestring(secretstr, &b);
 
 	if (result != ISC_R_SUCCESS) {
-		cfg_obj_log(key, ns_g_lctx, ISC_LOG_WARNING,
+		cfg_obj_log(key, named_g_lctx, ISC_LOG_WARNING,
 			    "secret for key '%s' on command channel: %s",
 			    keyid->keyname, isc_result_totext(result));
-		CHECK(result);
+		goto cleanup;
 	}
 
 	keyid->secret.length = isc_buffer_usedlength(&b);
 	keyid->secret.base = isc_mem_get(mctx,
 					 keyid->secret.length);
 	if (keyid->secret.base == NULL) {
-		cfg_obj_log(key, ns_g_lctx, ISC_LOG_WARNING,
+		cfg_obj_log(key, named_g_lctx, ISC_LOG_WARNING,
 			   "couldn't register key '%s': "
 			   "out of memory", keyid->keyname);
 		CHECK(ISC_R_NOMEMORY);
 	}
-	memcpy(keyid->secret.base, isc_buffer_base(&b),
-	       keyid->secret.length);
+	memmove(keyid->secret.base, isc_buffer_base(&b),
+		keyid->secret.length);
 	ISC_LIST_APPEND(*keyids, keyid, link);
 	keyid = NULL;
 	result = ISC_R_SUCCESS;
@@ -917,7 +973,7 @@ get_key_info(const cfg_obj_t *config, const cfg_obj_t *control,
 }
 
 static void
-update_listener(ns_controls_t *cp, controllistener_t **listenerp,
+update_listener(named_controls_t *cp, controllistener_t **listenerp,
 		const cfg_obj_t *control, const cfg_obj_t *config,
 		isc_sockaddr_t *addr, cfg_aclconfctx_t *aclconfctx,
 		const char *socktext, isc_sockettype_t type)
@@ -990,13 +1046,13 @@ update_listener(ns_controls_t *cp, controllistener_t **listenerp,
 		 * sake of avoiding this message would be too much trouble.
 		 */
 		if (control != NULL)
-			cfg_obj_log(control, ns_g_lctx, ISC_LOG_WARNING,
+			cfg_obj_log(control, named_g_lctx, ISC_LOG_WARNING,
 				    "couldn't install new keys for "
 				    "command channel %s: %s",
 				    socktext, isc_result_totext(result));
 		else
-			isc_log_write(ns_g_lctx, NS_LOGCATEGORY_GENERAL,
-				      NS_LOGMODULE_CONTROL, ISC_LOG_WARNING,
+			isc_log_write(named_g_lctx, NAMED_LOGCATEGORY_GENERAL,
+				      NAMED_LOGMODULE_CONTROL, ISC_LOG_WARNING,
 				      "couldn't install new keys for "
 				      "command channel %s: %s",
 				      socktext, isc_result_totext(result));
@@ -1007,32 +1063,40 @@ update_listener(ns_controls_t *cp, controllistener_t **listenerp,
 	 */
 	if (control != NULL && type == isc_sockettype_tcp) {
 		allow = cfg_tuple_get(control, "allow");
-		result = cfg_acl_fromconfig(allow, config, ns_g_lctx,
+		result = cfg_acl_fromconfig(allow, config, named_g_lctx,
 					    aclconfctx, listener->mctx, 0,
 					    &new_acl);
 	} else {
 		result = dns_acl_any(listener->mctx, &new_acl);
 	}
 
+	if (control != NULL) {
+		const cfg_obj_t *readonly;
+
+		readonly = cfg_tuple_get(control, "read-only");
+		if (!cfg_obj_isvoid(readonly))
+			listener->readonly = cfg_obj_asboolean(readonly);
+	}
+
 	if (result == ISC_R_SUCCESS) {
 		dns_acl_detach(&listener->acl);
 		dns_acl_attach(new_acl, &listener->acl);
 		dns_acl_detach(&new_acl);
 		/* XXXDCL say the old acl is still used? */
 	} else if (control != NULL)
-		cfg_obj_log(control, ns_g_lctx, ISC_LOG_WARNING,
+		cfg_obj_log(control, named_g_lctx, ISC_LOG_WARNING,
 			    "couldn't install new acl for "
 			    "command channel %s: %s",
 			    socktext, isc_result_totext(result));
 	else
-		isc_log_write(ns_g_lctx, NS_LOGCATEGORY_GENERAL,
-			      NS_LOGMODULE_CONTROL, ISC_LOG_WARNING,
+		isc_log_write(named_g_lctx, NAMED_LOGCATEGORY_GENERAL,
+			      NAMED_LOGMODULE_CONTROL, ISC_LOG_WARNING,
 			      "couldn't install new acl for "
 			      "command channel %s: %s",
 			      socktext, isc_result_totext(result));
 
 	if (result == ISC_R_SUCCESS && type == isc_sockettype_unix) {
-		isc_uint32_t perm, owner, group;
+		uint32_t perm, owner, group;
 		perm  = cfg_obj_asuint32(cfg_tuple_get(control, "perm"));
 		owner = cfg_obj_asuint32(cfg_tuple_get(control, "owner"));
 		group = cfg_obj_asuint32(cfg_tuple_get(control, "group"));
@@ -1046,7 +1110,7 @@ update_listener(ns_controls_t *cp, controllistener_t **listenerp,
 			listener->owner = owner;
 			listener->group = group;
 		} else if (control != NULL)
-			cfg_obj_log(control, ns_g_lctx, ISC_LOG_WARNING,
+			cfg_obj_log(control, named_g_lctx, ISC_LOG_WARNING,
 				    "couldn't update ownership/permission for "
 				    "command channel %s", socktext);
 	}
@@ -1055,7 +1119,7 @@ update_listener(ns_controls_t *cp, controllistener_t **listenerp,
 }
 
 static void
-add_listener(ns_controls_t *cp, controllistener_t **listenerp,
+add_listener(named_controls_t *cp, controllistener_t **listenerp,
 	     const cfg_obj_t *control, const cfg_obj_t *config,
 	     isc_sockaddr_t *addr, cfg_aclconfctx_t *aclconfctx,
 	     const char *socktext, isc_sockettype_t type)
@@ -1073,18 +1137,20 @@ add_listener(ns_controls_t *cp, controllistener_t **listenerp,
 		result = ISC_R_NOMEMORY;
 
 	if (result == ISC_R_SUCCESS) {
+		listener->mctx = NULL;
+		isc_mem_attach(mctx, &listener->mctx);
 		listener->controls = cp;
-		listener->mctx = mctx;
 		listener->task = cp->server->task;
 		listener->address = *addr;
 		listener->sock = NULL;
-		listener->listening = ISC_FALSE;
-		listener->exiting = ISC_FALSE;
+		listener->listening = false;
+		listener->exiting = false;
 		listener->acl = NULL;
 		listener->type = type;
 		listener->perm = 0;
 		listener->owner = 0;
 		listener->group = 0;
+		listener->readonly = false;
 		ISC_LINK_INIT(listener, link);
 		ISC_LIST_INIT(listener->keys);
 		ISC_LIST_INIT(listener->connections);
@@ -1094,7 +1160,7 @@ add_listener(ns_controls_t *cp, controllistener_t **listenerp,
 		 */
 		if (control != NULL && type == isc_sockettype_tcp) {
 			allow = cfg_tuple_get(control, "allow");
-			result = cfg_acl_fromconfig(allow, config, ns_g_lctx,
+			result = cfg_acl_fromconfig(allow, config, named_g_lctx,
 						    aclconfctx, mctx, 0,
 						    &new_acl);
 		} else {
@@ -1102,6 +1168,14 @@ add_listener(ns_controls_t *cp, controllistener_t **listenerp,
 		}
 	}
 
+	if ((result == ISC_R_SUCCESS) && (control != NULL)) {
+		const cfg_obj_t *readonly;
+
+		readonly = cfg_tuple_get(control, "read-only");
+		if (!cfg_obj_isvoid(readonly))
+			listener->readonly = cfg_obj_asboolean(readonly);
+	}
+
 	if (result == ISC_R_SUCCESS) {
 		dns_acl_attach(new_acl, &listener->acl);
 		dns_acl_detach(&new_acl);
@@ -1122,7 +1196,7 @@ add_listener(ns_controls_t *cp, controllistener_t **listenerp,
 			result = get_rndckey(mctx, &listener->keys);
 
 		if (result != ISC_R_SUCCESS && control != NULL)
-			cfg_obj_log(control, ns_g_lctx, ISC_LOG_WARNING,
+			cfg_obj_log(control, named_g_lctx, ISC_LOG_WARNING,
 				    "couldn't install keys for "
 				    "command channel %s: %s",
 				    socktext, isc_result_totext(result));
@@ -1139,15 +1213,20 @@ add_listener(ns_controls_t *cp, controllistener_t **listenerp,
 	}
 
 	if (result == ISC_R_SUCCESS && type == isc_sockettype_unix)
-		isc_socket_cleanunix(&listener->address, ISC_FALSE);
+		isc_socket_cleanunix(&listener->address, false);
 
 	if (result == ISC_R_SUCCESS)
-		result = isc_socket_create(ns_g_socketmgr,
+		result = isc_socket_create(named_g_socketmgr,
 					   isc_sockaddr_pf(&listener->address),
 					   type, &listener->sock);
 	if (result == ISC_R_SUCCESS)
 		isc_socket_setname(listener->sock, "control", NULL);
 
+#ifndef ISC_ALLOW_MAPPED
+	if (result == ISC_R_SUCCESS)
+		isc_socket_ipv6only(listener->sock, true);
+#endif
+
 	if (result == ISC_R_SUCCESS)
 		result = isc_socket_bind(listener->sock, &listener->address,
 					 ISC_SOCKET_REUSEADDRESS);
@@ -1169,24 +1248,24 @@ add_listener(ns_controls_t *cp, controllistener_t **listenerp,
 		result = control_accept(listener);
 
 	if (result == ISC_R_SUCCESS) {
-		isc_log_write(ns_g_lctx, NS_LOGCATEGORY_GENERAL,
-			      NS_LOGMODULE_CONTROL, ISC_LOG_NOTICE,
+		isc_log_write(named_g_lctx, NAMED_LOGCATEGORY_GENERAL,
+			      NAMED_LOGMODULE_CONTROL, ISC_LOG_NOTICE,
 			      "command channel listening on %s", socktext);
 		*listenerp = listener;
 
 	} else {
 		if (listener != NULL) {
-			listener->exiting = ISC_TRUE;
+			listener->exiting = true;
 			free_listener(listener);
 		}
 
 		if (control != NULL)
-			cfg_obj_log(control, ns_g_lctx, ISC_LOG_WARNING,
+			cfg_obj_log(control, named_g_lctx, ISC_LOG_WARNING,
 				    "couldn't add command channel %s: %s",
 				    socktext, isc_result_totext(result));
 		else
-			isc_log_write(ns_g_lctx, NS_LOGCATEGORY_GENERAL,
-				      NS_LOGMODULE_CONTROL, ISC_LOG_NOTICE,
+			isc_log_write(named_g_lctx, NAMED_LOGCATEGORY_GENERAL,
+				      NAMED_LOGMODULE_CONTROL, ISC_LOG_NOTICE,
 				      "couldn't add command channel %s: %s",
 				      socktext, isc_result_totext(result));
 
@@ -1197,8 +1276,8 @@ add_listener(ns_controls_t *cp, controllistener_t **listenerp,
 }
 
 isc_result_t
-ns_controls_configure(ns_controls_t *cp, const cfg_obj_t *config,
-		      cfg_aclconfctx_t *aclconfctx)
+named_controls_configure(named_controls_t *cp, const cfg_obj_t *config,
+			 cfg_aclconfctx_t *aclconfctx)
 {
 	controllistener_t *listener;
 	controllistenerlist_t new_listeners;
@@ -1251,14 +1330,14 @@ ns_controls_configure(ns_controls_t *cp, const cfg_obj_t *config,
 				addr = *cfg_obj_assockaddr(obj);
 				if (isc_sockaddr_getport(&addr) == 0)
 					isc_sockaddr_setport(&addr,
-							     NS_CONTROL_PORT);
+						     NAMED_CONTROL_PORT);
 
 				isc_sockaddr_format(&addr, socktext,
 						    sizeof(socktext));
 
-				isc_log_write(ns_g_lctx,
-					      NS_LOGCATEGORY_GENERAL,
-					      NS_LOGMODULE_CONTROL,
+				isc_log_write(named_g_lctx,
+					      NAMED_LOGCATEGORY_GENERAL,
+					      NAMED_LOGMODULE_CONTROL,
 					      ISC_LOG_DEBUG(9),
 					      "processing control channel %s",
 					      socktext);
@@ -1318,9 +1397,9 @@ ns_controls_configure(ns_controls_t *cp, const cfg_obj_t *config,
 				result = isc_sockaddr_frompath(&addr,
 						      cfg_obj_asstring(path));
 				if (result != ISC_R_SUCCESS) {
-					isc_log_write(ns_g_lctx,
-					      NS_LOGCATEGORY_GENERAL,
-					      NS_LOGMODULE_CONTROL,
+					isc_log_write(named_g_lctx,
+					      NAMED_LOGCATEGORY_GENERAL,
+					      NAMED_LOGMODULE_CONTROL,
 					      ISC_LOG_DEBUG(9),
 					      "control channel '%s': %s",
 					      cfg_obj_asstring(path),
@@ -1328,9 +1407,9 @@ ns_controls_configure(ns_controls_t *cp, const cfg_obj_t *config,
 					continue;
 				}
 
-				isc_log_write(ns_g_lctx,
-					      NS_LOGCATEGORY_GENERAL,
-					      NS_LOGMODULE_CONTROL,
+				isc_log_write(named_g_lctx,
+					      NAMED_LOGCATEGORY_GENERAL,
+					      NAMED_LOGMODULE_CONTROL,
 					      ISC_LOG_DEBUG(9),
 					      "processing control channel '%s'",
 					      cfg_obj_asstring(path));
@@ -1380,7 +1459,7 @@ ns_controls_configure(ns_controls_t *cp, const cfg_obj_t *config,
 				isc_sockaddr_fromin6(&addr,
 						     &in6addr_loopback, 0);
 			}
-			isc_sockaddr_setport(&addr, NS_CONTROL_PORT);
+			isc_sockaddr_setport(&addr, NAMED_CONTROL_PORT);
 
 			isc_sockaddr_format(&addr, socktext, sizeof(socktext));
 
@@ -1410,7 +1489,7 @@ ns_controls_configure(ns_controls_t *cp, const cfg_obj_t *config,
 	}
 
 	/*
-	 * ns_control_shutdown() will stop whatever is on the global
+	 * named_control_shutdown() will stop whatever is on the global
 	 * listeners list, which currently only has whatever sockaddrs
 	 * were in the previous configuration (if any) that do not
 	 * remain in the current configuration.
@@ -1427,16 +1506,16 @@ ns_controls_configure(ns_controls_t *cp, const cfg_obj_t *config,
 }
 
 isc_result_t
-ns_controls_create(ns_server_t *server, ns_controls_t **ctrlsp) {
+named_controls_create(named_server_t *server, named_controls_t **ctrlsp) {
 	isc_mem_t *mctx = server->mctx;
 	isc_result_t result;
-	ns_controls_t *controls = isc_mem_get(mctx, sizeof(*controls));
+	named_controls_t *controls = isc_mem_get(mctx, sizeof(*controls));
 
 	if (controls == NULL)
 		return (ISC_R_NOMEMORY);
 	controls->server = server;
 	ISC_LIST_INIT(controls->listeners);
-	controls->shuttingdown = ISC_FALSE;
+	controls->shuttingdown = false;
 	controls->symtab = NULL;
 	result = isccc_cc_createsymtab(&controls->symtab);
 	if (result != ISC_R_SUCCESS) {
@@ -1448,8 +1527,8 @@ ns_controls_create(ns_server_t *server, ns_controls_t **ctrlsp) {
 }
 
 void
-ns_controls_destroy(ns_controls_t **ctrlsp) {
-	ns_controls_t *controls = *ctrlsp;
+named_controls_destroy(named_controls_t **ctrlsp) {
+	named_controls_t *controls = *ctrlsp;
 
 	REQUIRE(ISC_LIST_EMPTY(controls->listeners));
 
