@@ -14,7 +14,7 @@ rm -f */named.memstats
 rm -f */named.run
 rm -f */named.stats
 rm -f dig.out*
-rm -f dnstap.out
+rm -f dnstap.out dnstap.hex
 rm -f dnstap.out.save
 rm -f fstrm_capture.out
 rm -f ns*/dnstap.out
