@@ -136,8 +136,8 @@ rdataset_to_sortedarray(dns_rdataset_t *set, isc_mem_t *mctx,
 }
 
 isc_result_t
-dns_dnssec_keyfromrdata(dns_name_t *name, dns_rdata_t *rdata, isc_mem_t *mctx,
-			dst_key_t **key)
+dns_dnssec_keyfromrdata(const dns_name_t *name, const dns_rdata_t *rdata,
+			isc_mem_t *mctx, dst_key_t **key)
 {
 	isc_buffer_t b;
 	isc_region_t r;
@@ -185,7 +185,7 @@ digest_sig(dst_context_t *ctx, bool downcase, dns_rdata_t *sigrdata,
 }
 
 isc_result_t
-dns_dnssec_sign(dns_name_t *name, dns_rdataset_t *set, dst_key_t *key,
+dns_dnssec_sign(const dns_name_t *name, dns_rdataset_t *set, dst_key_t *key,
 		isc_stdtime_t *inception, isc_stdtime_t *expire,
 		isc_mem_t *mctx, isc_buffer_t *buffer, dns_rdata_t *sigrdata)
 {
@@ -365,7 +365,7 @@ cleanup_signature:
 }
 
 isc_result_t
-dns_dnssec_verify2(dns_name_t *name, dns_rdataset_t *set, dst_key_t *key,
+dns_dnssec_verify2(const dns_name_t *name, dns_rdataset_t *set, dst_key_t *key,
 		   bool ignoretime, isc_mem_t *mctx,
 		   dns_rdata_t *sigrdata, dns_name_t *wild)
 {
@@ -374,7 +374,7 @@ dns_dnssec_verify2(dns_name_t *name, dns_rdataset_t *set, dst_key_t *key,
 }
 
 isc_result_t
-dns_dnssec_verify3(dns_name_t *name, dns_rdataset_t *set, dst_key_t *key,
+dns_dnssec_verify3(const dns_name_t *name, dns_rdataset_t *set, dst_key_t *key,
 		   bool ignoretime, unsigned int maxbits,
 		   isc_mem_t *mctx, dns_rdata_t *sigrdata, dns_name_t *wild)
 {
@@ -593,7 +593,7 @@ cleanup_struct:
 }
 
 isc_result_t
-dns_dnssec_verify(dns_name_t *name, dns_rdataset_t *set, dst_key_t *key,
+dns_dnssec_verify(const dns_name_t *name, dns_rdataset_t *set, dst_key_t *key,
 		  bool ignoretime, isc_mem_t *mctx,
 		  dns_rdata_t *sigrdata)
 {
@@ -733,7 +733,7 @@ syncdelete(dst_key_t *key, isc_stdtime_t now) {
 
 isc_result_t
 dns_dnssec_findzonekeys3(dns_db_t *db, dns_dbversion_t *ver,
-			 dns_dbnode_t *node, dns_name_t *name,
+			 dns_dbnode_t *node, const dns_name_t *name,
 			 const char *directory, isc_stdtime_t now,
 			 isc_mem_t *mctx, unsigned int maxkeys,
 			 dst_key_t **keys, unsigned int *nkeys)
@@ -896,7 +896,7 @@ dns_dnssec_findzonekeys3(dns_db_t *db, dns_dbversion_t *ver,
 
 isc_result_t
 dns_dnssec_findzonekeys2(dns_db_t *db, dns_dbversion_t *ver,
-			 dns_dbnode_t *node, dns_name_t *name,
+			 dns_dbnode_t *node, const dns_name_t *name,
 			 const char *directory, isc_mem_t *mctx,
 			 unsigned int maxkeys, dst_key_t **keys,
 			 unsigned int *nkeys)
@@ -910,9 +910,9 @@ dns_dnssec_findzonekeys2(dns_db_t *db, dns_dbversion_t *ver,
 
 isc_result_t
 dns_dnssec_findzonekeys(dns_db_t *db, dns_dbversion_t *ver,
-			dns_dbnode_t *node, dns_name_t *name, isc_mem_t *mctx,
-			unsigned int maxkeys, dst_key_t **keys,
-			unsigned int *nkeys)
+			dns_dbnode_t *node, const dns_name_t *name,
+			isc_mem_t *mctx, unsigned int maxkeys,
+			dst_key_t **keys, unsigned int *nkeys)
 {
 	isc_stdtime_t now;
 
@@ -1194,7 +1194,7 @@ failure:
  * Does this key ('rdata') self sign the rrset ('rdataset')?
  */
 bool
-dns_dnssec_selfsigns(dns_rdata_t *rdata, dns_name_t *name,
+dns_dnssec_selfsigns(dns_rdata_t *rdata, const dns_name_t *name,
 		     dns_rdataset_t *rdataset, dns_rdataset_t *sigrdataset,
 		     bool ignoretime, isc_mem_t *mctx)
 {
@@ -1214,7 +1214,7 @@ dns_dnssec_selfsigns(dns_rdata_t *rdata, dns_name_t *name,
 }
 
 bool
-dns_dnssec_signs(dns_rdata_t *rdata, dns_name_t *name,
+dns_dnssec_signs(dns_rdata_t *rdata, const dns_name_t *name,
 		     dns_rdataset_t *rdataset, dns_rdataset_t *sigrdataset,
 		     bool ignoretime, isc_mem_t *mctx)
 {
@@ -1410,7 +1410,7 @@ get_hints(dns_dnsseckey_t *key, isc_stdtime_t now) {
  * Get a list of DNSSEC keys from the key repository
  */
 isc_result_t
-dns_dnssec_findmatchingkeys2(dns_name_t *origin, const char *directory,
+dns_dnssec_findmatchingkeys2(const dns_name_t *origin, const char *directory,
 			     isc_stdtime_t now, isc_mem_t *mctx,
 			     dns_dnsseckeylist_t *keylist)
 {
@@ -1541,7 +1541,7 @@ dns_dnssec_findmatchingkeys2(dns_name_t *origin, const char *directory,
 }
 
 isc_result_t
-dns_dnssec_findmatchingkeys(dns_name_t *origin, const char *directory,
+dns_dnssec_findmatchingkeys(const dns_name_t *origin, const char *directory,
 			    isc_mem_t *mctx, dns_dnsseckeylist_t *keylist)
 {
 	isc_stdtime_t now;
@@ -1661,7 +1661,7 @@ mark_active_keys(dns_dnsseckeylist_t *keylist, dns_rdataset_t *rrsigs) {
  * Add the contents of a DNSKEY rdataset 'keyset' to 'keylist'.
  */
 isc_result_t
-dns_dnssec_keylistfromrdataset(dns_name_t *origin,
+dns_dnssec_keylistfromrdataset(const dns_name_t *origin,
 			       const char *directory, isc_mem_t *mctx,
 			       dns_rdataset_t *keyset, dns_rdataset_t *keysigs,
 			       dns_rdataset_t *soasigs, bool savekeys,
@@ -1863,7 +1863,7 @@ delrdata(dns_rdata_t *rdata, dns_diff_t *diff, dns_name_t *origin,
 }
 
 static isc_result_t
-publish_key(dns_diff_t *diff, dns_dnsseckey_t *key, dns_name_t *origin,
+publish_key(dns_diff_t *diff, dns_dnsseckey_t *key, const dns_name_t *origin,
 	    dns_ttl_t ttl, isc_mem_t *mctx, bool allzsk,
 	    void (*report)(const char *, ...))
 {
@@ -1905,7 +1905,7 @@ publish_key(dns_diff_t *diff, dns_dnsseckey_t *key, dns_name_t *origin,
 }
 
 static isc_result_t
-remove_key(dns_diff_t *diff, dns_dnsseckey_t *key, dns_name_t *origin,
+remove_key(dns_diff_t *diff, dns_dnsseckey_t *key, const dns_name_t *origin,
 	  dns_ttl_t ttl, isc_mem_t *mctx, const char *reason,
 	  void (*report)(const char *, ...))
 {
@@ -2080,7 +2080,7 @@ dns_dnssec_syncupdate(dns_dnsseckeylist_t *keys, dns_dnsseckeylist_t *rmkeys,
  */
 isc_result_t
 dns_dnssec_updatekeys(dns_dnsseckeylist_t *keys, dns_dnsseckeylist_t *newkeys,
-		      dns_dnsseckeylist_t *removed, dns_name_t *origin,
+		      dns_dnsseckeylist_t *removed, const dns_name_t *origin,
 		      dns_ttl_t hint_ttl, dns_diff_t *diff,
 		      bool allzsk, isc_mem_t *mctx,
 		      void (*report)(const char *, ...))
