@@ -1,25 +1,15 @@
 /*
- * Copyright (C) 2004-2014  Internet Systems Consortium, Inc. ("ISC")
- * Copyright (C) 1998-2003  Internet Software Consortium.
+ * Copyright (C) Internet Systems Consortium, Inc. ("ISC")
  *
- * Permission to use, copy, modify, and/or distribute this software for any
- * purpose with or without fee is hereby granted, provided that the above
- * copyright notice and this permission notice appear in all copies.
+ * This Source Code Form is subject to the terms of the Mozilla Public
+ * License, v. 2.0. If a copy of the MPL was not distributed with this
+ * file, You can obtain one at http://mozilla.org/MPL/2.0/.
  *
- * THE SOFTWARE IS PROVIDED "AS IS" AND ISC DISCLAIMS ALL WARRANTIES WITH
- * REGARD TO THIS SOFTWARE INCLUDING ALL IMPLIED WARRANTIES OF MERCHANTABILITY
- * AND FITNESS.  IN NO EVENT SHALL ISC BE LIABLE FOR ANY SPECIAL, DIRECT,
- * INDIRECT, OR CONSEQUENTIAL DAMAGES OR ANY DAMAGES WHATSOEVER RESULTING FROM
- * LOSS OF USE, DATA OR PROFITS, WHETHER IN AN ACTION OF CONTRACT, NEGLIGENCE
- * OR OTHER TORTIOUS ACTION, ARISING OUT OF OR IN CONNECTION WITH THE USE OR
- * PERFORMANCE OF THIS SOFTWARE.
+ * See the COPYRIGHT file distributed with this work for additional
+ * information regarding copyright ownership.
  */
 
-/* $Id$ */
-
-/*! \file
- * \author Principal Author: Bob Halley
- */
+/*! \file */
 
 /*
  * XXXRTH  Need to document the states a task can be in, and the rules
@@ -28,6 +18,8 @@
 
 #include <config.h>
 
+#include <stdbool.h>
+
 #include <isc/app.h>
 #include <isc/condition.h>
 #include <isc/event.h>
@@ -37,9 +29,11 @@
 #include <isc/msgs.h>
 #include <isc/once.h>
 #include <isc/platform.h>
+#include <isc/print.h>
 #include <isc/string.h>
 #include <isc/task.h>
 #include <isc/thread.h>
+#include <isc/time.h>
 #include <isc/util.h>
 #include <isc/xml.h>
 
@@ -65,8 +59,6 @@
 #define USE_SHARED_MANAGER
 #endif	/* ISC_PLATFORM_USETHREADS */
 
-#include "task_p.h"
-
 #ifdef ISC_TASK_TRACE
 #define XTRACE(m)		fprintf(stderr, "task %p thread %lu: %s\n", \
 				       task, isc_thread_self(), (m))
@@ -115,6 +107,7 @@ struct isc__task {
 	unsigned int			quantum;
 	unsigned int			flags;
 	isc_stdtime_t			now;
+	isc_time_t			tnow;
 	char				name[16];
 	void *				tag;
 	/* Locked by task manager lock. */
@@ -156,9 +149,16 @@ struct isc__taskmgr {
 #endif /* ISC_PLATFORM_USETHREADS */
 	unsigned int			tasks_running;
 	unsigned int			tasks_ready;
-	isc_boolean_t			pause_requested;
-	isc_boolean_t			exclusive_requested;
-	isc_boolean_t			exiting;
+	bool			pause_requested;
+	bool			exclusive_requested;
+	bool			exiting;
+
+	/*
+	 * Multiple threads can read/write 'excl' at the same time, so we need
+	 * to protect the access.  We can't use 'lock' since isc_task_detach()
+	 * will try to acquire it.
+	 */
+	isc_mutex_t			excl_lock;
 	isc__task_t			*excl;
 #ifdef USE_SHARED_MANAGER
 	unsigned int			refs;
@@ -196,7 +196,7 @@ isc__task_purgerange(isc_task_t *task0, void *sender, isc_eventtype_t first,
 unsigned int
 isc__task_purge(isc_task_t *task, void *sender, isc_eventtype_t type,
 		void *tag);
-isc_boolean_t
+bool
 isc_task_purgeevent(isc_task_t *task0, isc_event_t *event);
 unsigned int
 isc__task_unsendrange(isc_task_t *task, void *sender, isc_eventtype_t first,
@@ -220,6 +220,8 @@ void *
 isc__task_gettag(isc_task_t *task0);
 void
 isc__task_getcurrenttime(isc_task_t *task0, isc_stdtime_t *t);
+void
+isc__task_getcurrenttimex(isc_task_t *task0, isc_time_t *t);
 isc_result_t
 isc__taskmgr_create(isc_mem_t *mctx, unsigned int workers,
 		    unsigned int default_quantum, isc_taskmgr_t **managerp);
@@ -234,15 +236,15 @@ isc__task_beginexclusive(isc_task_t *task);
 void
 isc__task_endexclusive(isc_task_t *task0);
 void
-isc__task_setprivilege(isc_task_t *task0, isc_boolean_t priv);
-isc_boolean_t
+isc__task_setprivilege(isc_task_t *task0, bool priv);
+bool
 isc__task_privilege(isc_task_t *task0);
 void
 isc__taskmgr_setmode(isc_taskmgr_t *manager0, isc_taskmgrmode_t mode);
 isc_taskmgrmode_t
 isc__taskmgr_mode(isc_taskmgr_t *manager0);
 
-static inline isc_boolean_t
+static inline bool
 empty_readyq(isc__taskmgr_t *manager);
 
 static inline isc__task_t *
@@ -257,7 +259,8 @@ static struct isc__taskmethods {
 	/*%
 	 * The following are defined just for avoiding unused static functions.
 	 */
-	void *purgeevent, *unsendrange, *getname, *gettag, *getcurrenttime;
+	void *purgeevent, *unsendrange, *getname, *gettag,
+	     *getcurrenttime, *getcurrenttimex;
 } taskmethods = {
 	{
 		isc__task_attach,
@@ -280,7 +283,8 @@ static struct isc__taskmethods {
 	(void *)isc__task_unsendrange,
 	(void *)isc__task_getname,
 	(void *)isc__task_gettag,
-	(void *)isc__task_getcurrenttime
+	(void *)isc__task_getcurrenttime,
+	(void *)isc__task_getcurrenttimex
 };
 
 static isc_taskmgrmethods_t taskmgrmethods = {
@@ -335,7 +339,7 @@ isc__task_create(isc_taskmgr_t *manager0, unsigned int quantum,
 {
 	isc__taskmgr_t *manager = (isc__taskmgr_t *)manager0;
 	isc__task_t *task;
-	isc_boolean_t exiting;
+	bool exiting;
 	isc_result_t result;
 
 	REQUIRE(VALID_MANAGER(manager));
@@ -359,20 +363,21 @@ isc__task_create(isc_taskmgr_t *manager0, unsigned int quantum,
 	task->quantum = quantum;
 	task->flags = 0;
 	task->now = 0;
+	isc_time_settoepoch(&task->tnow);
 	memset(task->name, 0, sizeof(task->name));
 	task->tag = NULL;
 	INIT_LINK(task, link);
 	INIT_LINK(task, ready_link);
 	INIT_LINK(task, ready_priority_link);
 
-	exiting = ISC_FALSE;
+	exiting = false;
 	LOCK(&manager->lock);
 	if (!manager->exiting) {
 		if (task->quantum == 0)
 			task->quantum = manager->default_quantum;
 		APPEND(manager->tasks, task, link);
 	} else
-		exiting = ISC_TRUE;
+		exiting = true;
 	UNLOCK(&manager->lock);
 
 	if (exiting) {
@@ -409,9 +414,9 @@ isc__task_attach(isc_task_t *source0, isc_task_t **targetp) {
 	*targetp = (isc_task_t *)source;
 }
 
-static inline isc_boolean_t
+static inline bool
 task_shutdown(isc__task_t *task) {
-	isc_boolean_t was_idle = ISC_FALSE;
+	bool was_idle = false;
 	isc_event_t *event, *prev;
 
 	/*
@@ -427,7 +432,7 @@ task_shutdown(isc__task_t *task) {
 		if (task->state == task_state_idle) {
 			INSIST(EMPTY(task->events));
 			task->state = task_state_ready;
-			was_idle = ISC_TRUE;
+			was_idle = true;
 		}
 		INSIST(task->state == task_state_ready ||
 		       task->state == task_state_running);
@@ -457,7 +462,7 @@ static inline void
 task_ready(isc__task_t *task) {
 	isc__taskmgr_t *manager = task->manager;
 #ifdef USE_WORKER_THREADS
-	isc_boolean_t has_privilege = isc__task_privilege((isc_task_t *) task);
+	bool has_privilege = isc__task_privilege((isc_task_t *) task);
 #endif /* USE_WORKER_THREADS */
 
 	REQUIRE(VALID_MANAGER(manager));
@@ -474,7 +479,7 @@ task_ready(isc__task_t *task) {
 	UNLOCK(&manager->lock);
 }
 
-static inline isc_boolean_t
+static inline bool
 task_detach(isc__task_t *task) {
 
 	/*
@@ -497,16 +502,16 @@ task_detach(isc__task_t *task) {
 		 * loop to deal with shutting down and termination.
 		 */
 		task->state = task_state_ready;
-		return (ISC_TRUE);
+		return (true);
 	}
 
-	return (ISC_FALSE);
+	return (false);
 }
 
 void
 isc__task_detach(isc_task_t **taskp) {
 	isc__task_t *task;
-	isc_boolean_t was_idle;
+	bool was_idle;
 
 	/*
 	 * Detach *taskp from its task.
@@ -528,9 +533,9 @@ isc__task_detach(isc_task_t **taskp) {
 	*taskp = NULL;
 }
 
-static inline isc_boolean_t
+static inline bool
 task_send(isc__task_t *task, isc_event_t **eventp) {
-	isc_boolean_t was_idle = ISC_FALSE;
+	bool was_idle = false;
 	isc_event_t *event;
 
 	/*
@@ -542,11 +547,12 @@ task_send(isc__task_t *task, isc_event_t **eventp) {
 	REQUIRE(event != NULL);
 	REQUIRE(event->ev_type > 0);
 	REQUIRE(task->state != task_state_done);
+	REQUIRE(!ISC_LINK_LINKED(event, ev_ratelink));
 
 	XTRACE("task_send");
 
 	if (task->state == task_state_idle) {
-		was_idle = ISC_TRUE;
+		was_idle = true;
 		INSIST(EMPTY(task->events));
 		task->state = task_state_ready;
 	}
@@ -562,7 +568,7 @@ task_send(isc__task_t *task, isc_event_t **eventp) {
 void
 isc__task_send(isc_task_t *task0, isc_event_t **eventp) {
 	isc__task_t *task = (isc__task_t *)task0;
-	isc_boolean_t was_idle;
+	bool was_idle;
 
 	/*
 	 * Send '*event' to 'task'.
@@ -603,7 +609,7 @@ isc__task_send(isc_task_t *task0, isc_event_t **eventp) {
 
 void
 isc__task_sendanddetach(isc_task_t **taskp, isc_event_t **eventp) {
-	isc_boolean_t idle1, idle2;
+	bool idle1, idle2;
 	isc__task_t *task;
 
 	/*
@@ -640,7 +646,7 @@ isc__task_sendanddetach(isc_task_t **taskp, isc_event_t **eventp) {
 static unsigned int
 dequeue_events(isc__task_t *task, void *sender, isc_eventtype_t first,
 	       isc_eventtype_t last, void *tag,
-	       isc_eventlist_t *events, isc_boolean_t purging)
+	       isc_eventlist_t *events, bool purging)
 {
 	isc_event_t *event, *next_event;
 	unsigned int count = 0;
@@ -696,10 +702,11 @@ isc__task_purgerange(isc_task_t *task0, void *sender, isc_eventtype_t first,
 	ISC_LIST_INIT(events);
 
 	count = dequeue_events(task, sender, first, last, tag, &events,
-			       ISC_TRUE);
+			       true);
 
 	for (event = HEAD(events); event != NULL; event = next_event) {
 		next_event = NEXT(event, ev_link);
+		ISC_LIST_UNLINK(events, event, ev_link);
 		isc_event_free(&event);
 	}
 
@@ -723,7 +730,7 @@ isc__task_purge(isc_task_t *task, void *sender, isc_eventtype_t type,
 	return (isc__task_purgerange(task, sender, type, type, tag));
 }
 
-isc_boolean_t
+bool
 isc_task_purgeevent(isc_task_t *task0, isc_event_t *event) {
 	isc__task_t *task = (isc__task_t *)task0;
 	isc_event_t *curr_event, *next_event;
@@ -760,11 +767,11 @@ isc_task_purgeevent(isc_task_t *task0, isc_event_t *event) {
 	UNLOCK(&task->lock);
 
 	if (curr_event == NULL)
-		return (ISC_FALSE);
+		return (false);
 
 	isc_event_free(&curr_event);
 
-	return (ISC_TRUE);
+	return (true);
 }
 
 unsigned int
@@ -779,7 +786,7 @@ isc__task_unsendrange(isc_task_t *task, void *sender, isc_eventtype_t first,
 	XTRACE("isc_task_unsendrange");
 
 	return (dequeue_events((isc__task_t *)task, sender, first,
-			       last, tag, events, ISC_FALSE));
+			       last, tag, events, false));
 }
 
 unsigned int
@@ -793,7 +800,7 @@ isc__task_unsend(isc_task_t *task, void *sender, isc_eventtype_t type,
 	XTRACE("isc_task_unsend");
 
 	return (dequeue_events((isc__task_t *)task, sender, type,
-			       type, tag, events, ISC_FALSE));
+			       type, tag, events, false));
 }
 
 isc_result_t
@@ -801,7 +808,7 @@ isc__task_onshutdown(isc_task_t *task0, isc_taskaction_t action,
 		     void *arg)
 {
 	isc__task_t *task = (isc__task_t *)task0;
-	isc_boolean_t disallowed = ISC_FALSE;
+	bool disallowed = false;
 	isc_result_t result = ISC_R_SUCCESS;
 	isc_event_t *event;
 
@@ -824,7 +831,7 @@ isc__task_onshutdown(isc_task_t *task0, isc_taskaction_t action,
 
 	LOCK(&task->lock);
 	if (TASK_SHUTTINGDOWN(task)) {
-		disallowed = ISC_TRUE;
+		disallowed = true;
 		result = ISC_R_SHUTTINGDOWN;
 	} else
 		ENQUEUE(task->on_shutdown, event, ev_link);
@@ -839,7 +846,7 @@ isc__task_onshutdown(isc_task_t *task0, isc_taskaction_t action,
 void
 isc__task_shutdown(isc_task_t *task0) {
 	isc__task_t *task = (isc__task_t *)task0;
-	isc_boolean_t was_idle;
+	bool was_idle;
 
 	/*
 	 * Shutdown 'task'.
@@ -879,8 +886,7 @@ isc__task_setname(isc_task_t *task0, const char *name, void *tag) {
 	REQUIRE(VALID_TASK(task));
 
 	LOCK(&task->lock);
-	memset(task->name, 0, sizeof(task->name));
-	strncpy(task->name, name, sizeof(task->name) - 1);
+	strlcpy(task->name, name, sizeof(task->name));
 	task->tag = tag;
 	UNLOCK(&task->lock);
 }
@@ -915,18 +921,30 @@ isc__task_getcurrenttime(isc_task_t *task0, isc_stdtime_t *t) {
 	UNLOCK(&task->lock);
 }
 
+void
+isc__task_getcurrenttimex(isc_task_t *task0, isc_time_t *t) {
+	isc__task_t *task = (isc__task_t *)task0;
+
+	REQUIRE(VALID_TASK(task));
+	REQUIRE(t != NULL);
+
+	LOCK(&task->lock);
+	*t = task->tnow;
+	UNLOCK(&task->lock);
+}
+
 /***
  *** Task Manager.
  ***/
 
 /*
- * Return ISC_TRUE if the current ready list for the manager, which is
+ * Return true if the current ready list for the manager, which is
  * either ready_tasks or the ready_priority_tasks, depending on whether
  * the manager is currently in normal or privileged execution mode.
  *
  * Caller must hold the task manager lock.
  */
-static inline isc_boolean_t
+static inline bool
 empty_readyq(isc__taskmgr_t *manager) {
 	isc__tasklist_t queue;
 
@@ -935,7 +953,7 @@ empty_readyq(isc__taskmgr_t *manager) {
 	else
 		queue = manager->ready_priority_tasks;
 
-	return (ISC_TF(EMPTY(queue)));
+	return (EMPTY(queue));
 }
 
 /*
@@ -1082,9 +1100,9 @@ dispatch(isc__taskmgr_t *manager) {
 		task = pop_readyq(manager);
 		if (task != NULL) {
 			unsigned int dispatch_count = 0;
-			isc_boolean_t done = ISC_FALSE;
-			isc_boolean_t requeue = ISC_FALSE;
-			isc_boolean_t finished = ISC_FALSE;
+			bool done = false;
+			bool requeue = false;
+			bool finished = false;
 			isc_event_t *event;
 
 			INSIST(VALID_TASK(task));
@@ -1103,7 +1121,8 @@ dispatch(isc__taskmgr_t *manager) {
 			task->state = task_state_running;
 			XTRACE(isc_msgcat_get(isc_msgcat, ISC_MSGSET_GENERAL,
 					      ISC_MSG_RUNNING, "running"));
-			isc_stdtime_get(&task->now);
+			TIME_NOW(&task->tnow);
+			task->now = isc_time_seconds(&task->tnow);
 			do {
 				if (!EMPTY(task->events)) {
 					event = HEAD(task->events);
@@ -1133,7 +1152,7 @@ dispatch(isc__taskmgr_t *manager) {
 				if (task->references == 0 &&
 				    EMPTY(task->events) &&
 				    !TASK_SHUTTINGDOWN(task)) {
-					isc_boolean_t was_idle;
+					bool was_idle;
 
 					/*
 					 * There are no references and no
@@ -1180,11 +1199,11 @@ dispatch(isc__taskmgr_t *manager) {
 							       ISC_MSGSET_TASK,
 							       ISC_MSG_DONE,
 							       "done"));
-						finished = ISC_TRUE;
+						finished = true;
 						task->state = task_state_done;
 					} else
 						task->state = task_state_idle;
-					done = ISC_TRUE;
+					done = true;
 				} else if (dispatch_count >= task->quantum) {
 					/*
 					 * Our quantum has expired, but
@@ -1201,8 +1220,8 @@ dispatch(isc__taskmgr_t *manager) {
 							      ISC_MSG_QUANTUM,
 							      "quantum"));
 					task->state = task_state_ready;
-					requeue = ISC_TRUE;
-					done = ISC_TRUE;
+					requeue = true;
+					done = true;
 				}
 			} while (!done);
 			UNLOCK(&task->lock);
@@ -1315,6 +1334,7 @@ manager_free(isc__taskmgr_t *manager) {
 	isc_mem_free(manager->mctx, manager->threads);
 #endif /* USE_WORKER_THREADS */
 	DESTROYLOCK(&manager->lock);
+	DESTROYLOCK(&manager->excl_lock);
 	manager->common.impmagic = 0;
 	manager->common.magic = 0;
 	mctx = manager->mctx;
@@ -1367,6 +1387,11 @@ isc__taskmgr_create(isc_mem_t *mctx, unsigned int workers,
 	result = isc_mutex_init(&manager->lock);
 	if (result != ISC_R_SUCCESS)
 		goto cleanup_mgr;
+	result = isc_mutex_init(&manager->excl_lock);
+	if (result != ISC_R_SUCCESS) {
+		DESTROYLOCK(&manager->lock);
+		goto cleanup_mgr;
+	}
 
 #ifdef USE_WORKER_THREADS
 	manager->workers = 0;
@@ -1409,9 +1434,9 @@ isc__taskmgr_create(isc_mem_t *mctx, unsigned int workers,
 	INIT_LIST(manager->ready_priority_tasks);
 	manager->tasks_running = 0;
 	manager->tasks_ready = 0;
-	manager->exclusive_requested = ISC_FALSE;
-	manager->pause_requested = ISC_FALSE;
-	manager->exiting = ISC_FALSE;
+	manager->exclusive_requested = false;
+	manager->pause_requested = false;
+	manager->exiting = false;
 	manager->excl = NULL;
 
 	isc_mem_attach(mctx, &manager->mctx);
@@ -1425,6 +1450,10 @@ isc__taskmgr_create(isc_mem_t *mctx, unsigned int workers,
 		if (isc_thread_create(run, manager,
 				      &manager->threads[manager->workers]) ==
 		    ISC_R_SUCCESS) {
+			char name[16];	/* thread name limit on Linux */
+			snprintf(name, sizeof(name), "isc-worker%04u", i);
+			isc_thread_setname(manager->threads[manager->workers],
+					   name);
 			manager->workers++;
 			started++;
 		}
@@ -1499,8 +1528,10 @@ isc__taskmgr_destroy(isc_taskmgr_t **managerp) {
 	/*
 	 * Detach the exclusive task before acquiring the manager lock
 	 */
+	LOCK(&manager->excl_lock);
 	if (manager->excl != NULL)
 		isc__task_detach((isc_task_t **) &manager->excl);
+	UNLOCK(&manager->excl_lock);
 
 	/*
 	 * Unlike elsewhere, we're going to hold this lock a long time.
@@ -1517,7 +1548,7 @@ isc__taskmgr_destroy(isc_taskmgr_t **managerp) {
 	 * Make sure we only get called once.
 	 */
 	INSIST(!manager->exiting);
-	manager->exiting = ISC_TRUE;
+	manager->exiting = true;
 
 	/*
 	 * If privileged mode was on, turn it off.
@@ -1557,8 +1588,16 @@ isc__taskmgr_destroy(isc_taskmgr_t **managerp) {
 	UNLOCK(&manager->lock);
 	while (isc__taskmgr_ready((isc_taskmgr_t *)manager))
 		(void)isc__taskmgr_dispatch((isc_taskmgr_t *)manager);
-	if (!ISC_LIST_EMPTY(manager->tasks))
+	if (!ISC_LIST_EMPTY(manager->tasks)) {
+		isc__task_t *t;
 		isc_mem_printallactive(stderr);
+		for (t = ISC_LIST_HEAD(manager->tasks);
+		     t != NULL;
+		     t = ISC_LIST_NEXT(t, link))
+		{
+			fprintf(stderr, "task: %p (%s)\n", t, t->name);
+		}
+	}
 	INSIST(ISC_LIST_EMPTY(manager->tasks));
 #ifdef USE_SHARED_MANAGER
 	taskmgr = NULL;
@@ -1590,17 +1629,17 @@ isc__taskmgr_mode(isc_taskmgr_t *manager0) {
 }
 
 #ifndef USE_WORKER_THREADS
-isc_boolean_t
+bool
 isc__taskmgr_ready(isc_taskmgr_t *manager0) {
 	isc__taskmgr_t *manager = (isc__taskmgr_t *)manager0;
-	isc_boolean_t is_ready;
+	bool is_ready;
 
 #ifdef USE_SHARED_MANAGER
 	if (manager == NULL)
 		manager = taskmgr;
 #endif
 	if (manager == NULL)
-		return (ISC_FALSE);
+		return (false);
 
 	LOCK(&manager->lock);
 	is_ready = !empty_readyq(manager);
@@ -1629,11 +1668,11 @@ isc__taskmgr_dispatch(isc_taskmgr_t *manager0) {
 void
 isc__taskmgr_pause(isc_taskmgr_t *manager0) {
 	isc__taskmgr_t *manager = (isc__taskmgr_t *)manager0;
+	manager->pause_requested = true;
 	LOCK(&manager->lock);
 	while (manager->tasks_running > 0) {
 		WAIT(&manager->paused, &manager->lock);
 	}
-	manager->pause_requested = ISC_TRUE;
 	UNLOCK(&manager->lock);
 }
 
@@ -1643,7 +1682,7 @@ isc__taskmgr_resume(isc_taskmgr_t *manager0) {
 
 	LOCK(&manager->lock);
 	if (manager->pause_requested) {
-		manager->pause_requested = ISC_FALSE;
+		manager->pause_requested = false;
 		BROADCAST(&manager->work_available);
 	}
 	UNLOCK(&manager->lock);
@@ -1657,23 +1696,29 @@ isc_taskmgr_setexcltask(isc_taskmgr_t *mgr0, isc_task_t *task0) {
 
 	REQUIRE(VALID_MANAGER(mgr));
 	REQUIRE(VALID_TASK(task));
+	LOCK(&mgr->excl_lock);
 	if (mgr->excl != NULL)
 		isc__task_detach((isc_task_t **) &mgr->excl);
 	isc__task_attach(task0, (isc_task_t **) &mgr->excl);
+	UNLOCK(&mgr->excl_lock);
 }
 
 isc_result_t
 isc_taskmgr_excltask(isc_taskmgr_t *mgr0, isc_task_t **taskp) {
 	isc__taskmgr_t *mgr = (isc__taskmgr_t *) mgr0;
+	isc_result_t result = ISC_R_SUCCESS;
 
 	REQUIRE(VALID_MANAGER(mgr));
 	REQUIRE(taskp != NULL && *taskp == NULL);
 
-	if (mgr->excl == NULL)
-		return (ISC_R_NOTFOUND);
+	LOCK(&mgr->excl_lock);
+	if (mgr->excl != NULL)
+		isc__task_attach((isc_task_t *) mgr->excl, taskp);
+	else
+		result = ISC_R_NOTFOUND;
+	UNLOCK(&mgr->excl_lock);
 
-	isc__task_attach((isc_task_t *) mgr->excl, taskp);
-	return (ISC_R_SUCCESS);
+	return (result);
 }
 
 isc_result_t
@@ -1683,14 +1728,17 @@ isc__task_beginexclusive(isc_task_t *task0) {
 	isc__taskmgr_t *manager = task->manager;
 
 	REQUIRE(task->state == task_state_running);
-	/* XXX: Require task == manager->excl? */
+/*
+ *  TODO REQUIRE(task == task->manager->excl);
+ *  it should be here, it fails on shutdown server->task
+ */
 
 	LOCK(&manager->lock);
 	if (manager->exclusive_requested) {
 		UNLOCK(&manager->lock);
 		return (ISC_R_LOCKBUSY);
 	}
-	manager->exclusive_requested = ISC_TRUE;
+	manager->exclusive_requested = true;
 	while (manager->tasks_running > 1) {
 		WAIT(&manager->exclusive_granted, &manager->lock);
 	}
@@ -1710,7 +1758,7 @@ isc__task_endexclusive(isc_task_t *task0) {
 	REQUIRE(task->state == task_state_running);
 	LOCK(&manager->lock);
 	REQUIRE(manager->exclusive_requested);
-	manager->exclusive_requested = ISC_FALSE;
+	manager->exclusive_requested = false;
 	BROADCAST(&manager->work_available);
 	UNLOCK(&manager->lock);
 #else
@@ -1719,13 +1767,13 @@ isc__task_endexclusive(isc_task_t *task0) {
 }
 
 void
-isc__task_setprivilege(isc_task_t *task0, isc_boolean_t priv) {
+isc__task_setprivilege(isc_task_t *task0, bool priv) {
 	isc__task_t *task = (isc__task_t *)task0;
 	isc__taskmgr_t *manager = task->manager;
-	isc_boolean_t oldpriv;
+	bool oldpriv;
 
 	LOCK(&task->lock);
-	oldpriv = ISC_TF((task->flags & TASK_F_PRIVILEGED) != 0);
+	oldpriv = (task->flags & TASK_F_PRIVILEGED);
 	if (priv)
 		task->flags |= TASK_F_PRIVILEGED;
 	else
@@ -1745,13 +1793,13 @@ isc__task_setprivilege(isc_task_t *task0, isc_boolean_t priv) {
 	UNLOCK(&manager->lock);
 }
 
-isc_boolean_t
+bool
 isc__task_privilege(isc_task_t *task0) {
 	isc__task_t *task = (isc__task_t *)task0;
-	isc_boolean_t priv;
+	bool priv;
 
 	LOCK(&task->lock);
-	priv = ISC_TF((task->flags & TASK_F_PRIVILEGED) != 0);
+	priv = (task->flags & TASK_F_PRIVILEGED);
 	UNLOCK(&task->lock);
 	return (priv);
 }
@@ -1761,7 +1809,7 @@ isc__task_register(void) {
 	return (isc_task_register(isc__taskmgr_create));
 }
 
-isc_boolean_t
+bool
 isc_task_exiting(isc_task_t *t) {
 	isc__task_t *task = (isc__task_t *)t;
 
@@ -1939,7 +1987,7 @@ isc_taskmgr_renderjson(isc_taskmgr_t *mgr0, json_object *tasks) {
 		CHECKMEM(taskobj);
 		json_object_array_add(array, taskobj);
 
-		sprintf(buf, "%p", task);
+		snprintf(buf, sizeof(buf), "%p", task);
 		obj = json_object_new_string(buf);
 		CHECKMEM(obj);
 		json_object_object_add(taskobj, "id", obj);
@@ -2233,7 +2281,7 @@ isc_task_endexclusive(isc_task_t *task) {
 }
 
 void
-isc_task_setprivilege(isc_task_t *task, isc_boolean_t priv) {
+isc_task_setprivilege(isc_task_t *task, bool priv) {
 	REQUIRE(ISCAPI_TASK_VALID(task));
 
 	if (isc_bind9)
@@ -2242,7 +2290,7 @@ isc_task_setprivilege(isc_task_t *task, isc_boolean_t priv) {
 		task->methods->setprivilege(task, priv);
 }
 
-isc_boolean_t
+bool
 isc_task_privilege(isc_task_t *task) {
 	REQUIRE(ISCAPI_TASK_VALID(task));
 
@@ -2260,6 +2308,14 @@ isc_task_getcurrenttime(isc_task_t *task, isc_stdtime_t *t) {
 	isc__task_getcurrenttime(task, t);
 }
 
+void
+isc_task_getcurrenttimex(isc_task_t *task, isc_time_t *t) {
+	if (!isc_bind9)
+		return;
+
+	isc__task_getcurrenttimex(task, t);
+}
+
 /*%
  * This is necessary for libisc's internal timer implementation.  Other
  * implementation might skip implementing this.
